/*
 * iAirZoning
 * Copyright 2020 José Antonio Jiménez Campos
 */

#ifndef __IAIRZONING_HEADER_H__
#define __IAIRZONING_HEADER_H__

// Version
#define FIRMWARE_VERSION                    "1.0.0"
#define FIRMWARE_VERSION_OCTAL              010000

// Task Stack Sizes
#define INIT_TASK_SIZE                      (configMINIMAL_STACK_SIZE * 3)
#define READ_SENSORS_TASK_SIZE              (configMINIMAL_STACK_SIZE * 2)
#define OPEN_CLOSE_GATE_TASK_SIZE           (configMINIMAL_STACK_SIZE * 2)
#define IR_TX_TASK_SIZE                     (configMINIMAL_STACK_SIZE * 2)

// Task Priorities
#define INIT_TASK_PRIORITY                  (tskIDLE_PRIORITY + 2)
#define READ_SENSORS_TASK_PRIORITY          (tskIDLE_PRIORITY + 1)
#define OPEN_CLOSE_GATE_TASK_PRIORITY       (tskIDLE_PRIORITY + 2)
#define IR_TX_TASK_PRIORITY                 (configMAX_PRIORITIES - 1)

// WiFi Status
#define WIFI_STATUS_DISCONNECTED            0
#define WIFI_STATUS_CONNECTING              1
#define WIFI_STATUS_CONNECTED_STAGE_1       2
#define WIFI_STATUS_CONNECTED_STAGE_2       3

// HomeKit Thermostat Status
#define THERMOSTAT_MODE_OFF                 0
#define THERMOSTAT_MODE_IDLE                1
#define THERMOSTAT_MODE_HEATER              2
#define THERMOSTAT_MODE_COOLER              3
#define THERMOSTAT_TARGET_MODE_AUTO         0
#define THERMOSTAT_TARGET_MODE_HEATER       1
#define THERMOSTAT_TARGET_MODE_COOLER       2

// Thermostat Modes
#define THERMOSTAT_OFF                      0
#define THERMOSTAT_HEATER_ON                1
#define THERMOSTAT_HEATER_SOFT_ON           2
#define THERMOSTAT_HEATER_IDLE              3
#define THERMOSTAT_HEATER_FORCE_IDLE        4
#define THERMOSTAT_COOLER_ON                5
#define THERMOSTAT_COOLER_SOFT_ON           6
#define THERMOSTAT_COOLER_IDLE              7
#define THERMOSTAT_COOLER_FORCE_IDLE        8

#define SET_ZONE_SETTINGS_DELAY_MS          5000
#define UPDATE_TH_DELAY_MS                  250

// Gate Status
#define THERMOSTAT_GATE_CLOSED              0
#define THERMOSTAT_GATE_OPENED              1
#define THERMOSTAT_GATE_MUST_BE_CLOSED      2
#define THERMOSTAT_GATE_MUST_BE_OPENED      3
#define THERMOSTAT_GATE_CLOSING             4
#define THERMOSTAT_GATE_OPENING             5

// A/C Main modes
#define AC_MAIN_MODE_OFF                    0
#define AC_MAIN_MODE_HEATER                 1
#define AC_MAIN_MODE_COOLER                 2

// A/C IR Modes
#define AC_MAIN_IR_OFF                      0
#define AC_MAIN_IR_HEATER_ON                1
#define AC_MAIN_IR_HEATER_SOFT_ON           2
#define AC_MAIN_IR_HEATER_IDLE              3
#define AC_MAIN_IR_HEATER_FORCE_IDLE        4
#define AC_MAIN_IR_COOLER_ON                5
#define AC_MAIN_IR_COOLER_SOFT_ON           6
#define AC_MAIN_IR_COOLER_IDLE              7
#define AC_MAIN_IR_COOLER_FORCE_IDLE        8

// WiFi WatchDog Parameters
#define WIFI_WATCHDOG_POLL_PERIOD_MS        5500

// Custom Macros
#define MS_TO_TICK(x)                       ((x) / portTICK_PERIOD_MS)

// UART Output
#define INFO(message, ...)              printf(message "\n", ##__VA_ARGS__);
#define ERROR(message, ...)             INFO("! " message, ##__VA_ARGS__);

#endif // __IAIRZONING_HEADER_H__
