/*
* iAirZoning
* Copyright 2020 José Antonio Jiménez Campos
*/

#ifndef __IAIRZONING_IRDA_H__
#define __IAIRZONING_IRDA_H__

/*
 Prepara el GPIO encargado de manejar el LED Ir
 @param gpio        Número de GPIO donde está conectado el LED Ir
 @param inverted    Indica si el comportamiento del LED Ir es inverso al estado del GPIO
 @param freq        Frecuencia en KHz de las marcas de la yrama Ir
 */
void ir_tx_init(const uint8_t gpio, const bool inverted, const uint8_t freq);

/*
 Prepara el comando IrDA para ser enviado posteriormente por la tarea ir_tx_task.
 @param command         Puntero del array de uint8_t que codifica el comando a enviar
 @param command_len     Tamaño del array command
 */
void ir_tx_send(uint8_t* command, const uint16_t command_len);

/*
 Establece el protocolo IrDA a usar
 @param header_mark     Tiempo de duración en microsegundos de la marca de la cabecera
 @param header_space    Tiempo de duración en microsegundos del espacio de la cabecera
 @param bit0_mark       Tiempo de duración en microsegundos de la marca del bit 0
 @param bit0_space      Tiempo de duración en microsegundos del espacio del bit 0
 @param bit1_mark       Tiempo de duración en microsegundos de la marca del bit 1
 @param bit1_space      Tiempo de duración en microsegundos del espacio del bit 1
 @param footer_mark     Tiempo de duración en microsegundos de la marca del final de la trama Ir
 @param packet_len      Tamaño de la trama Ir
 */
void ir_tx_set_protocol(const uint16_t header_mark, const uint16_t header_space,
                        const uint16_t bit0_mark, const uint16_t bit0_space,
                        const uint16_t bit1_mark, const uint16_t bit1_space,
                        const uint16_t footer_mark,
                        const uint16_t packet_len);

#endif  // __IAIRZONING_IRDA_H__
