/*
 * iAirZoning
 * Copyright 2020 José Antonio Jiménez Campos
 */

#ifndef __MCP_23017_H__
#define __MCP_23017_H__

#define MCP_CHANNEL_A   0
#define MCP_CHANNEL_B   1

/*
 Función de inicialización de la interfaz MCP 23017 usando el bus I2C
 @param i2c_bus         Identificador del bus I2C a usar (0, 1, 2...)
 @param scl_pin         Pin SCL usado para el bus I2C
 @param sda_pin         Pin SDA usado para el bus I2C
 @param init_a_outs     Estado inicial de los GPIO del canal A de la interfaz MSP23017
 @param init_b_outs     Estado inicial de los GPIO del canal B de la interfaz MSP23017
 @return                Número de error. Devuelve 0 si OK.
 */
int mcp_init(const uint8_t i2c_bus, const uint8_t scl_pin, const uint8_t sda_pin, const uint8_t init_a_outs, const uint8_t init_b_outs);

/*
 Función para actuar sobre los GPIO correspondientes de la interfaz MCP 23017
 @param channel         Canal del GPIO sobre el que se va a actuar (0: canal A, 1: canal B)
 @param gpio            GPIO sobre el que se va a actuar
 @param value           Valor a establecer en el GPIO (0: LOW, 1: HIGH)
 @return                Número de error. Devuelve 0 si OK.
 */
int mcp_gpio_write(const uint8_t channel, const uint8_t gpio, const bool value);

#endif // __MCP_23017_H__
